/***************************************************************************/
/*
 * Audio_id.c
 *
 * Matches wavs to audio id enums
 */
/***************************************************************************/

#include "frame.h"
#include "audio_id.h"

/***************************************************************************/

typedef struct AUDIO_ID
{
	SDWORD	iID;
	STRING	*pWavStr;
}
AUDIO_ID;

/***************************************************************************/

static AUDIO_ID asAudioID[] =
{

/* Beeps */

{ID_SOUND_WINDOWCLOSE,					"Beep1.wav"},			
{ID_SOUND_WINDOWOPEN,					"Beep2.wav"},			
{ID_SOUND_SELECT,						"Beep4.wav"},			
{ID_SOUND_BUTTON_CLICK_5,				"Beep5.wav"},		
{FE_AUDIO_MESSAGEEND,					"Beep6.wav"},		
{ID_SOUND_ZOOM_ON_RADAR,					"Beep7.wav"},			
{ID_SOUND_BUILD_FAIL,					"Beep8.wav"},			
{ID_SOUND_MESSAGEEND,					"Beep9.wav"},		
{ID_SOUND_GAME_SHUTDOWN,					"GmeShtDn.wav"},		

/* Design Sequence */

{ID_SOUND_TURRET_SELECTED,				"PCV331.wav"},
{ID_SOUND_BODY_SELECTED,					"PCV332.wav"},
{ID_SOUND_PROPULSION_SELECTED,			"PCV333.wav"},
{ID_SOUND_DESIGN_COMPLETED,				"PCV334.wav"},

/* Structures */

{ID_SOUND_CONSTRUCTION_STARTED,			"PCV335.wav"}, 
{ID_SOUND_STRUCTURE_COMPLETED,			"PCV336.wav"}, 	
{ID_SOUND_STRUCTURE_UNDER_ATTACK,		"PCV337.wav"}, 
{ID_SOUND_STRUCTURE_REPAIR_IN_PROGRESS,	"PCV339.wav"}, 
{ID_SOUND_STRUCTURE_DEMOLISHED,			"PCV340.wav"}, 

/* Power */

{ID_SOUND_POWER_GENERATOR_UNDER_ATTACK,	"PCV341.wav"}, 
{ID_SOUND_POWER_GENERATOR_DESTROYED,		"PCV342.wav"}, 
{ID_SOUND_POWER_LOW,						"PCV343.wav"}, 
{ID_SOUND_RESOURCE_HERE,					"PCV344.wav"}, 
{ID_SOUND_DERRICK_UNDER_ATTACK,			"PCV345.wav"}, 
{ID_SOUND_DERRICK_DESTROYED,				"PCV346.wav"}, 
{ID_SOUND_RESOURCE_DEPLETED,				"PCV347.wav"}, 
{ID_SOUND_POWER_TRANSFER_IN_PROGRESS,	"PCV348.wav"}, 
{ID_SOUND_POWER_GENERATOR_REQUIRED,		"PCV349.wav"}, 

/* Research */

{ID_SOUND_RESEARCH_FACILITY_REQUIRED,	"PCV350.wav"}, 
{ID_SOUND_ARTIFACT,						"PCV351.wav"}, 
{ID_SOUND_ARTIFACT_RECOVERED,			"PCV352.wav"}, 
{ID_SOUND_NEW_RESEARCH_PROJ_AVAILABLE,	"PCV353.wav"}, 
{ID_SOUND_NEW_STRUCTURE_AVAILABLE,		"PCV354.wav"}, 
{ID_SOUND_NEW_COMPONENT_AVAILABLE,		"PCV355.wav"}, 
{ID_SOUND_NEW_CYBORG_AVAILABLE,			"PCV356.wav"}, 
{ID_SOUND_RESEARCH_COMPLETED,			"PCV357.wav"}, 
{ID_SOUND_MAJOR_RESEARCH,				"PCV358.wav"}, 
{ID_SOUND_STRUCTURE_RESEARCH_COMPLETED,	"PCV359.wav"}, 
{ID_SOUND_POWER_RESEARCH_COMPLETED,		"PCV360.wav"}, 
{ID_SOUND_COMPUTER_RESEARCH_COMPLETED,	"PCV361.wav"}, 
{ID_SOUND_VEHICLE_RESEARCH_COMPLETED,	"PCV362.wav"}, 
{ID_SOUND_SYSTEMS_RESEARCH_COMPLETED,	"PCV363.wav"}, 
{ID_SOUND_WEAPON_RESEARCH_COMPLETED,		"PCV364.wav"}, 
{ID_SOUND_CYBORG_RESEARCH_COMPLETED,		"PCV365.wav"}, 

/* Production */

{ID_SOUND_PRODUCTION_STARTED,			"PCV366.wav"}, 
{ID_SOUND_DROID_COMPLETED,				"PCV367.wav"}, 
{ID_SOUND_PRODUCTION_PAUSED,				"PCV368.wav"}, 
{ID_SOUND_PRODUCTION_CANCELLED,			"PCV369.wav"}, 
{ID_SOUND_DELIVERY_POINT_ASSIGNED,		"PCV370.wav"}, 
{ID_SOUND_DELIVERY_POINT_ASSIGNED_TO,	"PCV371.wav"}, 

/* Repair */

{ID_SOUND_UNIT_REPAIRED,					"PCV372.wav"},

/* Detection */

{ID_SOUND_SCAVENGERS_DETECTED,			"PCV373.wav"}, 
{ID_SOUND_SCAVENGER_BASE_DETECTED,		"PCV374.wav"}, 
{ID_SOUND_SCAVENGER_OUTPOST_DETECTED,	"PCV375.wav"}, 
{ID_SOUND_POWER_RESOURCE,				"PCV376.wav"}, 
{ID_SOUND_ARTEFACT_DISC,					"PCV377.wav"}, 
{ID_SOUND_ENEMY_UNIT_DETECTED,			"PCV378.wav"}, 
{ID_SOUND_ENEMY_BASE_DETECTED,			"PCV379.wav"}, 
{ID_SOUND_ALLY_DETECTED,					"PCV380.wav"}, 
{ID_SOUND_ENEMY_TRANSPORT_DETECTED,		"PCV381.wav"}, 
{ID_SOUND_ENEMY_LZ_DETECTED,				"PCV382.wav"}, 
{ID_SOUND_FRIENDLY_LZ_DETECTED,			"PCV383.wav"}, 
{ID_SOUND_NEXUS_TOWER_DETECTED,			"PCV384.wav"}, 
{ID_SOUND_NEXUS_TURRET_DETECTED,			"PCV385.wav"}, 
{ID_SOUND_NEXUS_UNIT_DETECTED,			"PCV386.wav"}, 
{ID_SOUND_ENEMY_BATTERY_DETECTED,		"PCV387.wav"}, 
{ID_SOUND_ENEMY_VTOLS_DETECTED,			"PCV388.wav"}, 

/* Status */

{ID_SOUND_SCAVENGER_BASE,				"PCV389.wav"}, 
{ID_SOUND_SCAVENGER_OUTPOST,				"PCV390.wav"}, 
{ID_SOUND_SCAVENGER_OUTPOST_ERADICATED,	"PCV391.wav"}, 
{ID_SOUND_SCAVENGER_BASE_ERADICATED,		"PCV392.wav"}, 
{ID_SOUND_ENEMY_BASE,					"PCV393.wav"}, 
{ID_SOUND_ENEMY_BASE_ERADICATED,			"PCV394.wav"}, 
{ID_SOUND_INCOMING_ENEMY_TRANSPORT,		"PCV395.wav"}, 
{ID_SOUND_ENEMY_LZ,						"PCV396.wav"}, 
{ID_SOUND_LZ1,							"PCV397.wav"}, 
{ID_SOUND_LZ2,							"PCV398.wav"}, 

/* Combat */

{ID_SOUND_UNIT_UNDER_ATTACK,				"PCV399.wav"}, 
{ID_SOUND_UNIT_DESTROYED,				"PCV400.wav"}, 
{ID_SOUND_UNIT_RETREATING,				"PCV401.wav"}, 
{ID_SOUND_UNIT_RETURNING_FOR_REPAIR,		"PCV402.wav"}, 

/* Artillery Batteries */
{ID_SOUND_ASSIGNED_TO_SENSOR,			"PCV403.wav"}, 
{ID_SOUND_SENSOR_LOCKED_ON,				"PCV404.wav"}, 
{ID_SOUND_ASSIGNED_TO_COUNTER_RADAR,		"PCV405.wav"},	
{ID_SOUND_ENEMY_BATTERY_LOCATED,			"PCV406.wav"}, 
{ID_SOUND_BATTERY_FIRING_COUNTER_ATTACK,	"PCV407.wav"}, 

/* Vtols */

{ID_SOUND_INTERCEPTORS_LAUNCHED,			"PCV408.wav"}, 
{ID_SOUND_REARMING,						"PCV409.wav"}, 
{ID_SOUND_VTOLS_ENGAGING,				"PCV410.wav"}, 
{ID_SOUND_ASSIGNED,						"PCV411.wav"}, 
{ID_SOUND_INTERCEPTORS_ASSIGNED,			"PCV412.wav"}, 

/* Command Console */
{ID_SOUND_COMMAND_CONSOLE_ACTIVATED,		"PCV413.wav"}, 
{ID_SOUND_SHORT_RANGE,					"PCV414.wav"}, 
{ID_SOUND_LONG_RANGE,					"PCV415.wav"}, 
{ID_SOUND_OPTIMUM_RANGE,					"PCV416.wav"}, 
{ID_SOUND_RETREAT_AT_MEDIUM_DAMAGE,		"PCV417.wav"}, 
{ID_SOUND_RETREAT_AT_HEAVY_DAMAGE,		"PCV418.wav"}, 
{ID_SOUND_NO_RETREAT,					"PCV419.wav"}, 
{ID_SOUND_FIRE_AT_WILL,					"PCV420.wav"}, 
{ID_SOUND_RETURN_FIRE,					"PCV421.wav"}, 
{ID_SOUND_CEASEFIRE,						"PCV422.wav"}, 
{ID_SOUND_HOLD_POSITION,					"PCV423.wav"}, 
{ID_SOUND_GUARD,							"PCV424.wav"}, 
{ID_SOUND_PURSUE,						"PCV425.wav"}, 
{ID_SOUND_PATROL,						"PCV426.wav"}, 
{ID_SOUND_RETURN_TO_LZ,					"PCV427.wav"}, 
{ID_SOUND_RECYCLING,						"PCV428.wav"}, 
{ID_SOUND_SCATTER,						"PCV429.wav"}, 

/* Tutorial Stuff */

{ID_SOUND_NOT_POSSIBLE_TRY_AGAIN,		"PCV430.wav"}, 
{ID_SOUND_NO,							"PCV431.wav"}, 
{ID_SOUND_THAT_IS_INCORRECT,				"PCV432.wav"}, 
{ID_SOUND_WELL_DONE,						"PCV433.wav"}, 
{ID_SOUND_EXCELLENT,						"PCV434.wav"}, 

/* Group and Commander Assignment */

{ID_SOUND_ASSIGNED_TO_COMMANDER,			"PCV435.wav"}, 
{ID_SOUND_GROUP_REPORTING,				"PCV436.wav"}, 
{ID_SOUND_COMMANDER_REPORTING,			"PCV437.wav"}, 

/* Routing */

{ID_SOUND_ROUTE_OBSTRUCTED,				"PCV438.wav"}, 
{ID_SOUND_NO_ROUTE_AVAILABLE,			"PCV439.wav"}, 

/* Transports and LZS */
{ID_SOUND_REINFORCEMENTS_AVAILABLE,		"PCV440.wav"}, 
{ID_SOUND_REINFORCEMENTS_IN_TRANSIT,		"PCV441.wav"}, 
{ID_SOUND_TRANSPORT_LANDING,				"PCV442.wav"}, 
{ID_SOUND_TRANSPORT_UNDER_ATTACK,		"PCV443.wav"}, 
{ID_SOUND_TRANSPORT_REPAIRING,			"PCV444.wav"}, 
{ID_SOUND_LZ_COMPROMISED,				"PCV445.wav"}, 
{ID_SOUND_LZ_CLEAR,						"LZ-Clear.wav"}, 
{ID_SOUND_TRANSPORT_RETURNING_TO_BASE, 	"PCV446.wav"}, 
{ID_SOUND_TRANSPORT_UNABLE_TO_LAND, 		"PCV447.wav"},

/* Mission Messages */

{ID_SOUND_MISSION_OBJECTIVE,				"PCV448.wav"}, 
{ID_SOUND_MISSION_UPDATE,				"PCV449.wav"}, 
{ID_SOUND_WARZONE_PAUSED,				"PCV450.wav"}, 
{ID_SOUND_WARZONE_ACTIVE,				"PCV451.wav"}, 
{ID_SOUND_MISSION_RESULTS,				"PCV452.wav"}, 
{ID_SOUND_RESEARCH_STOLEN,				"PCV453.wav"}, 
{ID_SOUND_TECHNOLOGY_TAKEN,				"PCV454.wav"}, 
{ID_SOUND_INCOMING_TRANSMISSION,			"PCV455.wav"}, 
{ID_SOUND_INCOMING_INTELLIGENCE_REPORT,	"PCV456.wav"}, 
{ID_SOUND_MISSION_FAILED,				"PCV458.wav"}, 
{ID_SOUND_MISSION_SUCCESSFUL,			"PCV459.wav"}, 
{ID_SOUND_OBJECTIVE_ACCOMPLISHED,		"PCV460.wav"}, 
{ID_SOUND_OBJECTIVE_FAILED,				"PCV461.wav"}, 
{ID_SOUND_MISSION_TIMER_ACTIVATED,		"PCV462.wav"}, 
{ID_SOUND_10_MINUTES_REMAINING,			"PCV463.wav"}, 
{ID_SOUND_5_MINUTES_REMAINING,			"PCV464.wav"}, 
{ID_SOUND_3_MINUTES_REMAINING,			"PCV465.wav"}, 
{ID_SOUND_2_MINUTES_REMAINING,			"PCV466.wav"}, 
{ID_SOUND_1_MINUTE_REMAINING,			"PCV467.wav"}, 
{ID_SOUND_UNIT_CAPTURED,					"PCV468.wav"}, 
{ID_SOUND_SYSTEM_FAILURE_IMMINENT,		"PCV469.wav"}, 
{ID_SOUND_YOU_ARE_DEFEATED,				"PCV470.wav"}, 
{ID_SOUND_MISSILE_CODES_DECIPHERED,		"PCV471.wav"}, 
{ID_SOUND_1ST_MISSILE_CODES_DECIPHERED,	"PCV472.wav"}, 
{ID_SOUND_2ND_MISSILE_CODES_DECIPHERED,	"PCV473.wav"}, 
{ID_SOUND_3RD_MISSILE_CODES_DECIPHERED,	"PCV474.wav"}, 
{ID_SOUND_MISSILE_CODES_CRACKED,			"PCV475.wav"}, 
{ID_SOUND_ENTERING_WARZONE,				"PCV476.wav"}, 
{ID_ALLIANCE_ACC,						"PCV477.wav"}, 
{ID_ALLIANCE_BRO,						"PCV478.wav"}, 
{ID_ALLIANCE_OFF,						"PCV479.wav"}, 
{ID_CLAN_ENTER,							"PCV480.wav"}, 
{ID_CLAN_EXIT,							"PCV481.wav"}, 
{ID_GIFT,								"PCV482.wav"}, 
{ID_POWER_TRANSMIT,						"PCV483.wav"}, 
{ID_SENSOR_DOWNLOAD,						"PCV484.wav"}, 
{ID_TECHNOLOGY_TRANSFER,					"PCV485.wav"}, 
{ID_UNITS_TRANSFER,						"PCV486.wav"}, 

/* Group and Commander Voices - Male */
{ID_SOUND_GROUP,							"Group.wav"},
{ID_SOUND_GROUP_0,						"0.wav"},
{ID_SOUND_GROUP_1,						"1.wav"},
{ID_SOUND_GROUP_2,						"2.wav"},
{ID_SOUND_GROUP_3,						"3.wav"},
{ID_SOUND_GROUP_4,						"4.wav"},
{ID_SOUND_GROUP_5,						"5.wav"},
{ID_SOUND_GROUP_6,						"6.wav"},
{ID_SOUND_GROUP_7,						"7.wav"},
{ID_SOUND_GROUP_8,						"8.wav"},
{ID_SOUND_GROUP_9,						"9.wav"},
{ID_SOUND_REPORTING,						"Reprting.wav"},
{ID_SOUND_COMMANDER,						"Commnder.wav"},
{ID_SOUND_COM_SCAVS_DETECTED,			"COM021.wav"}, 
{ID_SOUND_COM_SCAV_BASE_DETECTED,		"COM022.wav"}, 
{ID_SOUND_COM_SCAV_OUTPOST_DETECTED,		"COM023.wav"}, 
{ID_SOUND_COM_RESOURCE_DETECTED,			"COM024.wav"}, 
{ID_SOUND_COM_ARTEFACT_DETECTED,			"COM025.wav"}, 
{ID_SOUND_COM_ENEMY_DETECTED,			"COM026.wav"}, 
{ID_SOUND_COM_ENEMY_BASE_DETECTED,		"COM027.wav"}, 
{ID_SOUND_COM_ALLY_DETECTED,				"COM028.wav"}, 
{ID_SOUND_COM_ENEMY_TRANSPORT_DETECTED,	"COM029.wav"}, 
{ID_SOUND_COM_ENEMY_LZ_DETECTED,			"COM030.wav"}, 
{ID_SOUND_COM_FRIENDLY_LZ_DETECTED,		"COM031.wav"}, 
{ID_SOUND_COM_NEXUS_TOWER_DETECTED,		"COM032.wav"}, 
{ID_SOUND_COM_NEXUS_TURRET_DETECTED,		"COM033.wav"}, 
{ID_SOUND_COM_NEXUS_DETECTED,			"COM034.wav"}, 
{ID_SOUND_COM_ENEMY_BATTERY_DETECTED,	"COM035.wav"}, 
{ID_SOUND_COM_ENEMY_VTOLS_DETECTED,		"COM036.wav"}, 
{ID_SOUND_COM_ROUTE_OBSTRUCTED,			"COM037.wav"}, 
{ID_SOUND_COM_NO_ROUTE_AVAILABLE,		"COM038.wav"}, 
{ID_SOUND_COM_UNABLE_TO_COMPLY,			"COM039.wav"}, 
{ID_SOUND_COM_RETURNING_FOR_REPAIR,		"COM040.wav"}, 
{ID_SOUND_COM_HEADING_FOR_RALLY_POINT,	"COM041.wav"}, 

/* Radio Clicks */
{ID_SOUND_RADIOCLICK_1,					"RadClik1.wav"},
{ID_SOUND_RADIOCLICK_2,					"RadClik2.wav"},
{ID_SOUND_RADIOCLICK_3,					"RadClik3.wav"},
{ID_SOUND_RADIOCLICK_4,					"RadClik4.wav"},
{ID_SOUND_RADIOCLICK_5,					"RadClik5.wav"},
{ID_SOUND_RADIOCLICK_6,					"RadClik6.wav"},

/* Transport Pilots */	
{ID_SOUND_APPROACHING_LZ,				"T-AproLZ.wav"}, 
{ID_SOUND_ALRIGHT_BOYS,					"T-ARBoys.wav"}, 
{ID_SOUND_GREEN_LIGHT_IN_5,				"T-GrnLi5.wav"}, 
{ID_SOUND_GREEN_LIGHT_IN_4,				"T-GrnLi4.wav"}, 
{ID_SOUND_GREEN_LIGHT_IN_3,				"T-GrnLi3.wav"}, 
{ID_SOUND_GREEN_LIGHT_IN_2,				"T-GrnLi2.wav"}, 
{ID_SOUND_GO_GO_GO,						"T-GoGoGo,wav"}, 
{ID_SOUND_PREPARE_FOR_DUST_OFF,			"T-DustOf.wav"}, 

/* VTol Pilots */
	
	/* Ver-1 */
{ID_SOUND_ENEMY_LOCATED1,				"V-Eloc1.wav"},  
{ID_SOUND_ON_OUR_WAY1,					"V-OnWay1.wav"}, 
{ID_SOUND_RETURNING_TO_BASE1,			"V-RetBa1.wav"}, 
{ID_SOUND_LOCKED_ON1,					"V-LocOn1.wav"}, 
{ID_SOUND_COMMENCING_ATTACK_RUN1,		"V-AtkRn1.wav"}, 
{ID_SOUND_ABORTING_ATTACK_RUN1,			"V-AbtRn1.wav"}, 

	/* Ver-2 */
{ID_SOUND_ENEMY_LOCATED2,				"V-Eloc2.wav"},  
{ID_SOUND_ON_OUR_WAY2,					"V-OnWay2.wav"}, 
{ID_SOUND_RETURNING_TO_BASE2,			"V-RetBa2.wav"}, 
{ID_SOUND_LOCKED_ON2,					"V-LocOn2.wav"}, 
{ID_SOUND_COMMENCING_ATTACK_RUN2,		"V-AtkRn2.wav"}, 
{ID_SOUND_ABORTING_ATTACK_RUN2,			"V-AbtRn2.wav"}, 

	/* Ver-3 */
{ID_SOUND_ENEMY_LOCATED3,				"V-Eloc3.wav"},  
{ID_SOUND_ON_OUR_WAY3,					"V-OnWay3.wav"}, 
{ID_SOUND_RETURNING_TO_BASE3,			"V-RetBa3.wav"}, 
{ID_SOUND_LOCKED_ON3,					"V-LocOn3.wav"}, 
{ID_SOUND_COMMENCING_ATTACK_RUN3,		"V-AtkRn3.wav"}, 
{ID_SOUND_ABORTING_ATTACK_RUN3,			"V-AbtRn3.wav"}, 


/* The Collective */
{ID_SOUND_COLL_CLEANSE_AND_DESTROY,		"COl011a.wav"},
{ID_SOUND_COLL_DESTROYING_BIOLOGICALS,	"COl012a.wav"}, 
{ID_SOUND_COLL_ATTACK,					"COl013a.wav"}, 
{ID_SOUND_COLL_FIRE,						"COl014a.wav"}, 
{ID_SOUND_COLL_ENEMY_DETECTED,			"COl015a.wav"}, 
{ID_SOUND_COLL_ENGAGING,					"COl016a.wav"}, 
{ID_SOUND_COLL_STARTING_ATTACK_RUN,		"COl017a.wav"}, 
{ID_SOUND_COLL_DIE,						"COl018a.wav"}, 
{ID_SOUND_COLL_INTERCEPT_AND_DESTROY,	"COl019a.wav"}, 
{ID_SOUND_COLL_ENEMY_DESTROYED,			"COl020a.wav"}, 

/* SFX */
	/* Weapon Sounds */			
{ID_SOUND_ROCKET,						"rocket.wav"},
{ID_SOUND_ROTARY_LASER,					"RotLsr.wav"},		
{ID_SOUND_GAUSSGUN,						"GaussGun.wav"},	
{ID_SOUND_LARGE_CANNON,					"LrgCan.wav"},		
{ID_SOUND_SMALL_CANNON,					"SmlCan.wav"},		
{ID_SOUND_MEDIUM_CANNON,					"MedCan.wav"},		
{ID_SOUND_FLAME_THROWER,					"FlmThrow.wav"},		
{ID_SOUND_PULSE_LASER,					"PlsLsr.wav"},		
{ID_SOUND_BEAM_LASER,					"BemLsr.wav"},		
{ID_SOUND_MORTAR,						"Mortar.wav"},	
{ID_SOUND_HOWITZ_FLIGHT,					"HwtzFlgt.wav"},		
{ID_SOUND_BABA_MG_1,						"MgBar1.wav"},
{ID_SOUND_BABA_MG_2,						"MgBar2.wav"},
{ID_SOUND_BABA_MG_3,						"MgBar3.wav"},
{ID_SOUND_BABA_MG_HEAVY,					"MgHeavy.wav"},
{ID_SOUND_BABA_MG_TOWER,					"MgTower.wav"},
{ID_SOUND_SPLASH,						"Splash.wav"},
{ID_SOUND_ASSAULT_MG,					"AsltMG.wav"},
{ID_SOUND_RAPID_CANNON,					"RapdCan.wav"},
{ID_SOUND_HIVEL_CANNON,					"HiVelCan.wav"},
{ID_SOUND_NEXUS_TOWER,					"NxsTower.wav"},

	/* Construction sounds */
{ID_SOUND_WELD_1,						"Weld-1.wav"},
{ID_SOUND_WELD_2,						"Weld-2.wav"},
{ID_SOUND_CONSTRUCTION_START,			"BldStart.wav"},
{ID_SOUND_CONSTRUCTION_LOOP,				"BldLoop.wav"},
{ID_SOUND_CONSTRUCTION_1,				"Build1.wav"},
{ID_SOUND_CONSTRUCTION_2,				"Build2.wav"},
{ID_SOUND_CONSTRUCTION_3,				"Build3.wav"},
{ID_SOUND_CONSTRUCTION_4,				"Build4.wav"},

	/* Explosions */

{ID_SOUND_EXPLOSION_SMALL,				"SmlExpl.wav"},	
{ID_SOUND_EXPLOSION_LASER,				"LsrExpl.wav"},	
{ID_SOUND_EXPLOSION,						"LrgExpl.wav"},	
{ID_SOUND_EXPLOSION_ANTITANK,			"ATnkExpl.wav"},	
{ID_SOUND_RICOCHET_1,					"Richet1.wav"},	
{ID_SOUND_RICOCHET_2,					"Richet2.wav"},	
{ID_SOUND_RICOCHET_3,					"Richet3.wav"},	
{ID_SOUND_BARB_SQUISH,					"Squish.wav"},	
{ID_SOUND_BUILDING_FALL,					"BldFall.wav"},
{ID_SOUND_NEXUS_EXPLOSION,				"NxsExpld.wav"},

	/* Droid Engine Noises */
{ID_SOUND_CONSTRUCTOR_MOVEOFF,			"Con-Move Off.wav"},
{ID_SOUND_CONSTRUCTOR_MOVE,				"Con-Move.wav"},
{ID_SOUND_CONSTRUCTOR_SHUTDOWN,			"Con-Shut down.wav"},
	
	/* Transports */
{ID_SOUND_BLIMP_FLIGHT,					"TFlight.wav"},	
{ID_SOUND_BLIMP_IDLE,					"THover.wav"},	
{ID_SOUND_BLIMP_LAND,					"TLand.wav"},		
{ID_SOUND_BLIMP_TAKE_OFF,				"TStart.wav"},	

	/* Vtols */
{ID_SOUND_VTOL_LAND,						"VtolLand.wav"}, 	
{ID_SOUND_VTOL_OFF,						"VtolOff.wav"}, 	
{ID_SOUND_VTOL_MOVE,						"VtolMove.wav"}, 	

	/* Treads */
{ID_SOUND_TREAD,							"Tread.wav"},

	/* Hovers */
{ID_SOUND_HOVER_MOVE,					"HovMove.wav"}, 	
{ID_SOUND_HOVER_START,					"HovStart.wav"},	 	
{ID_SOUND_HOVER_STOP,					"HovStop.wav"},	 	

	/* Cyborgs */
{ID_SOUND_CYBORG_MOVE,					"Cyber-Move.wav"},

	/* Buildings */

{ID_SOUND_OIL_PUMP_2,					"OilPump.wav"},
{ID_SOUND_POWER_HUM,						"PowerHum.wav"},
{ID_SOUND_POWER_SPARK,					"PowerSpk.wav"},	
{ID_SOUND_STEAM,							"Steam.wav"},	
{ID_SOUND_ECM_TOWER,						"ECMTower.wav"},
{ID_SOUND_FIRE_ROAR,						"FreRoar.wav"},	

	/* Misc */
	
{ID_SOUND_HELP,							"help.wav"},
{ID_SOUND_BARB_SCREAM,					"Scream.wav"},
{ID_SOUND_BARB_SCREAM2,					"Scream2.wav"},
{ID_SOUND_BARB_SCREAM3,					"Scream3.wav"},
{ID_SOUND_OF_SILENCE,					"Silence.wav"},
{ID_SOUND_NOFAULTS,						"Scream4.wav"},

	/* Extra */
{ID_SOUND_LANDING_ZONE,					"LndgZne.wav"},
{ID_SOUND_SATELLITE_UPLINK,				"Pcv652.wav"},
{ID_SOUND_NASDA_CENTRAL,					"Pcv653.wav"},
{ID_SOUND_NUCLEAR_REACTOR,				"Pcv654.wav"},
{ID_SOUND_SAM_SITE,						"Pcv655.wav"},
{ID_SOUND_MISSILE_SILO,					"Pcv656.wav"},
{ID_SOUND_MISSILE_NME_DETECTED,			"NmeDeted.wav"},
{ID_SOUND_STRUCTURE_CAPTURED,			"Pcv611.wav"},
{ID_SOUND_CIVILIAN_RESCUED,				"Pcv612.wav"},
{ID_SOUND_CIVILIANS_RESCUED,				"Pcv613.wav"},
{ID_SOUND_UNITS_RESCUED,					"Pcv615.wav"},
{ID_SOUND_GROUP_RESCUED,					"Pcv616.wav"},
{ID_SOUND_GROUP_CAPTURED,				"Pcv618.wav"},
{ID_SOUND_OBJECTIVE_CAPTURED,			"Pcv621.wav"},
{ID_SOUND_OBJECTIVE_DESTROYED,			"Pcv622.wav"},
{ID_SOUND_STRUCTURE_INFECTED,			"Pcv623.wav"},
{ID_SOUND_GROUP_INFECTED,				"Pcv625.wav"},
{ID_SOUND_OUT_OF_TIME,					"Pcv629.wav"},
{ID_SOUND_ENEMY_ESCAPED,					"Pcv631.wav"},
{ID_SOUND_ENEMY_ESCAPING,				"Pcv632.wav"},
{ID_SOUND_ENEMY_TRANSPORT_LANDING,		"Pcv633.wav"},
{ID_SOUND_TEAM_ALPHA_ERADICATED,			"Pcv635.wav"},
{ID_SOUND_TEAM_BETA_ERADICATED,			"Pcv636.wav"},
{ID_SOUND_TEAM_GAMMA_ERADICATED,			"Pcv637.wav"},
{ID_SOUND_TEAM_ALPHA_RESCUED,			"Pcv638.wav"},
{ID_SOUND_TEAM_BETA_RESCUED,				"Pcv639.wav"},
{ID_SOUND_TEAM_GAMMA_RESCUED,			"Pcv640.wav"},
{ID_SOUND_LASER_SATELLITE_FIRING,		"Pcv650.wav"},
{ID_SOUND_INCOMING_LASER_SAT_STRIKE,		"Pcv657.wav"},

	/* Nexus */
{ID_SOUND_NEXUS_DEFENCES_ABSORBED,		"DefAbsrd.wav"},
{ID_SOUND_NEXUS_DEFENCES_NEUTRALISED,	"DefNut.wav"},
{ID_SOUND_NEXUS_LAUGH1,					"Laugh1.wav"},
{ID_SOUND_NEXUS_LAUGH2,					"Laugh2.wav"},
{ID_SOUND_NEXUS_LAUGH3,					"Laugh3.wav"},
{ID_SOUND_NEXUS_PRODUCTION_COMPLETED,	"PordComp.wav"},
{ID_SOUND_NEXUS_RESEARCH_ABSORBED,		"ResAbsrd.wav"},
{ID_SOUND_NEXUS_STRUCTURE_ABSORBED,		"StrutAbs.wav"},
{ID_SOUND_NEXUS_STRUCTURE_NEUTRALISED,	"StrutNut.wav"},
{ID_SOUND_NEXUS_SYNAPTIC_LINK,			"SynpLnk.wav"},
{ID_SOUND_NEXUS_UNIT_ABSORBED,			"UntAbsrd.wav"},
{ID_SOUND_NEXUS_UNIT_NEUTRALISED,		"UntNut.wav"},

/* extra multiplayer sounds */
{ID_SOUND_CYBORG_GROUND,					"CybGrnd.wav"},
{ID_SOUND_CYBORG_HEAVY,					"HvCybMov.wav"},
{ID_SOUND_EMP,							"EMP.wav"},
{ID_SOUND_LASER_HEAVY,					"HevLsr.wav"},
{ID_SOUND_PLASMA_FLAMER,					"PlasFlm.wav"},
{ID_SOUND_UPLINK,						"UpLink.wav"},
{ID_SOUND_LAS_SAT_COUNTDOWN,             "LasStrk.wav"},
};

/***************************************************************************/

BOOL
audioID_GetIDFromStr( STRING *pWavStr, SDWORD *piID )
{
	SDWORD		i;

	for ( i=0; i<ID_MAX_SOUND; i++ )
	{
		if ( stricmp( pWavStr, asAudioID[i].pWavStr ) == 0 )
		{
			ASSERT( (i == asAudioID[i].iID,
				"audioID_GetIDFromStr: %s stored IDs don't match", pWavStr ) );

			*piID = asAudioID[i].iID;
			return TRUE;
		}
	}

	*piID = NO_SOUND;
	return FALSE;
}

/***************************************************************************/
